// localStorage key name;
import Cookies from 'js-cookie';
export const LOCAL_LANG = 'lang';
export const LOCAL_AUTH = 'ut';
export function setCookie (name, value, domain) {
    Cookies.set(name, value, { domain });
}
export function getCookie (name) {
    return Cookies.get(name);
}

export function delCookie (name, domain) {
    Cookies.remove(name, { domain });
}
export function setLocalAuth (auth, domain) {
    // window.localStorage[LOCAL_AUTH] = auth;
    Cookies.set(LOCAL_AUTH, auth, { domain });
}

export function delLocalAuth (domain) {
    // window.localStorage.removeItem(LOCAL_AUTH);
    Cookies.remove(LOCAL_AUTH, { domain });
}

export function getLocalAuth () {
    // return window.localStorage[LOCAL_AUTH];
    return Cookies.get(LOCAL_AUTH);
}

export function setLocalLang (lang, domain) {
   // window.localStorage[LOCAL_LANG] = auth;
    Cookies.set(LOCAL_LANG, lang, { domain });
}

export function delLocalLang (domain) {
    // window.localStorage.removeItem(LOCAL_LANG);
    Cookies.remove(LOCAL_LANG, { domain });
}

export function getLocalLang () {
    // return window.localStorage[LOCAL_LANG];
    return Cookies.get(LOCAL_LANG);
}

// 判断参数是否是其中之一
export function oneOf (value, validList) {
    for (let i = 0; i < validList.length; i++) {
        if (value === validList[i]) {
            return true;
        }
    }
    return false;
}
/**
 * 判断参数是否是其中之一，并且返回index
 * @param value
 * @param validList
 * @return {number}
 */
export function oneOfIndex (value, validList) {
    for (let i = 0; i < validList.length; i++) {
        if (value === validList[i]) {
            return i;
        }
    }
    return -1;
}

/**
 * 从数据里删除元素
 * @param val
 * @param sourceList
 * @param fieldName
 */
export function removeFromArray (val, sourceList) {
    const index = oneOfIndex(val, sourceList);
    if (index > -1) {
        sourceList.splice(index, 1);
    }
}

/**
 * 将url的参数转换为Object对象
 * @param url
 */
export function urlParam2Obj (url) {
    if (url.indexOf('?') <= -1) {
        return {};
    }
    const search = url.split('?')[1];
    let params = {};
    if (search) {
        params = JSON.parse('{"' + search.replace(/"/g, '\\"').replace(/&/g, '","').replace(/=/g, '":"') + '"}');
    }
    let reKeys = Object.keys(params);
    for (let i = 0; i < reKeys.length; i++) {
        let keyName = reKeys[i];
        params[keyName] = decodeURIComponent(params[keyName]);
    }
    return params;
}

/**
 * 将backUrl，params 拼接成url的参数
 * @param url
 * @param backUrl
 * @param params
 * @return {string}
 */
export function encodeJumpUrl (url, params) {
    let reParams = params;
    if (isEmpty(reParams)) {
        reParams = {};
    }
    let urlParams = '?';
    let reKeys = Object.keys(reParams);
    for (let i = 0; i < reKeys.length; i++) {
        if (i !== 0) {
            urlParams += `&`;
        }
        urlParams += `${reKeys[i]}=` + encodeURIComponent(reParams[reKeys[i]]);
    }
    return url + urlParams;
}

/**
 * 判断对旬是否为空
 */
export function isEmpty (val) {
    if (val === null || typeof val === 'undefined') {
        return true;
    }
    if (typeof val === 'string') {
        if (trim(val) === '') {
            return true;
        }
    }
    if (Object.prototype.toString.call(val) === '[object Array]') {
        if (val.length === 0) {
            return true;
        }
    }
    return false;
}

/**
 * 去除两端的空格
 * @param str
 * @returns {string}
 */
export function trim (str) {
    if (str) {
        return str.replace(/^\s*/, '').replace(/\s*$/, '');
    }
    return '';
}

/**
 * 保留小数数
 * 解决js Number 处理浮点数精度的问题
 * @param val
 * @param point
 */
export function toFixed (val, point) {
    let changeNum = (parseInt(val * Math.pow(10, point) + 0.5) / Math.pow(10, point)).toString();
    let index = changeNum.indexOf('.');
    if (index < 0 && point > 0) {
        changeNum = changeNum + '.';
        for (let i = 0; i < point; i++) {
            changeNum = changeNum + '0';
        }
    } else {
        index = changeNum.length - index;
        for (let i = 0; i < (point - index) + 1; i++) {
            changeNum = changeNum + '0';
        }
    }
    return changeNum;
}

/**
 * deep clone
 * @param source
 * @return {*}
 */
export function deepClone (source) {
    if (!source && typeof source !== 'object') {
        throw new Error('error arguments', 'shallowClone');
    }
    const targetObj = source.constructor === Array ? [] : {};
    for (const keys in source) {
        if (source.hasOwnProperty(keys)) {
            if (source[keys] && typeof source[keys] === 'object') {
                targetObj[keys] = source[keys].constructor === Array ? [] : {};
                targetObj[keys] = deepClone(source[keys]);
            } else {
                targetObj[keys] = source[keys];
            }
        }
    }
    return targetObj;
}

/**
 * 对日期进行格式化，
 * dateFormat(new Date(), 'yyyy-MM-dd hh:mm:ss TT','en-US','');
 * @param time 要格式化的日期,日期对象或日期时间戳
 * @param format 进行格式化的模式字符串 默认:'dd MMM yyyy'
 *     支持的模式字母有：
 *     y:年,
 *     M:年中的月份(1-12),
 *     MMM:短月份
 *     MMMM:长月份
 *     d:月份中的天(1-31),
 *     ddd:短星期
 *     dddd:长星期
 *     h:小时(0-23),
 *     m:分(0-59),
 *     s:秒(0-59),
 *     tt:am/pm
 *     TT:AM/PM
 *     S:毫秒(0-999),
 *     q:季度(1-4)
 * @param nullStr 当为空时返回的内容
 * @param locale 语言
 *     目前支持的有：
 *     en-US: 美语
 *     id-ID: 印尼语
 * @return String
 */
export function dateFormat (time, formatV, nullStr, locale) {
    if (arguments.length === 0 || isEmpty(time)) {
        return nullStr || '';
    }
    let date;
    if ((time + '').length === 10) {
        date = +time * 1000;
    }
    if (typeof time === 'object') {
        date = time;
    } else {
        date = new Date(parseInt(time));
    }
    let zeroize = function (value, length) {
        if (!length) length = 2;
        value = String(value);
        for (var i = 0, zeros = ''; i < (length - value.length); i++) {
            zeros += '0';
        }
        return zeros + value;
    };
    let dateLocale = {
        'en-US': {
            month: {
                long: ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'],
                short: ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
            },
            week: {
                long: ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'],
                short: ['Sun', 'Mon', 'Tue', 'Wed', 'Thr', 'Fri', 'Sat']
            }
        },
        'id-ID': {
            month: {
                long: ['Januari', 'Februari', 'Maret', 'April', 'Mei', 'Juni', 'Juli', 'Agustus', 'September', 'Oktober', 'November', 'Desember'],
                short: ['Jan', 'Feb', 'Mar', 'Apr', 'Mei', 'Jun', 'Jul', 'Agu', 'Sep', 'Okt', 'Nov', 'Des']
            },
            week: {
                long: ['Minggu', 'Senin', 'Selasa', 'Rabu', 'Kamis', 'Jumat', 'Sabtu'],
                short: ['Min', 'Sen', 'Sel', 'Rab', 'Kam', 'Jum', 'Sab']
            }
        }

    };

    let dateStyle = dateLocale[ locale || getLocalLang() ];
    let formatStr = formatV || 'dd MMM yyyy';

    return formatStr.replace(/"[^"]*"|'[^']*'|\b(?:d{1,4}|M{1,4}|yy(?:yy)?|([hHmstT])\1?|[lLqZ])\b/g, function ($0) {
        switch ($0) {
            case 'd': return date.getDate();
            case 'dd': return zeroize(date.getDate());
            case 'ddd': return dateStyle.week.short[date.getDay()];
            case 'dddd': return dateStyle.week.long[date.getDay()];
            case 'M': return date.getMonth() + 1;
            case 'MM': return zeroize(date.getMonth() + 1);
            case 'MMM': return dateStyle.month.short[date.getMonth()];
            case 'MMMM': return dateStyle.month.long[date.getMonth()];
            case 'yy': return String(date.getFullYear()).substr(2);
            case 'yyyy': return date.getFullYear();
            case 'h': return date.getHours() % 12 || 12;
            case 'hh': return zeroize(date.getHours() % 12 || 12);
            case 'H': return date.getHours();
            case 'HH': return zeroize(date.getHours());
            case 'm': return date.getMinutes();
            case 'mm': return zeroize(date.getMinutes());
            case 's': return date.getSeconds();
            case 'ss': return zeroize(date.getSeconds());
            case 'q': return Math.floor((date.getMonth()+3)/3);
            case 'l': return zeroize(date.getMilliseconds(), 3);
            case 'L':
                let m = date.getMilliseconds();
                if (m > 99) m = Math.round(m / 10);
                return zeroize(m);
            case 'tt': return date.getHours() < 12 ? 'am' : 'pm';
            case 'TT': return date.getHours() < 12 ? 'AM' : 'PM';
            case 'Z': return date.toUTCString().match(/[A-Z]+$/);
            default: return $0.substr(1, $0.length - 2);
        }
    });
}

export function hasScrollY (el) {
    // TODO 需要解决兼容性问题，不同浏览器下处理结果不一样
    console.log('el.scrollHeight', el.scrollHeight, 'el.offsetHeight', el.offsetHeight, 'el.clientHeight', el.clientHeight);
    console.log('el.scrollHeight', document.body.scrollHeight, 'el.offsetHeight', document.body.offsetHeight, 'el.clientHeight', document.body.clientHeight);

    // test targets
    return el.scrollHeight > el.clientHeight;
    /*
    let elems = el ? [el] : [document.documentElement, document.body];
    let scrollY = false;
    for (let i = 0; i < elems.length; i++) {
        let o = elems[i];
        // test vertical
        let st = o.scrollTop;
        o.scrollTop += (st > 0) ? -1 : 1;
        o.scrollTop !== st && (scrollY = scrollY || true);
        o.scrollTop = st;
    }

    return scrollY;
    */
}

export function hasScrollX (el) {
    // TODO 需要解决兼容性问题，不同浏览器下处理结果不一样
    console.log('el.scrollHeight', el.scrollHeight, 'el.clientHeight', el.clientHeight);
    let elems = el ? [el] : [document.documentElement, document.body];
    let scrollX = false;
    for (let i = 0; i < elems.length; i++) {
        let o = elems[i];
        let sl = o.scrollLeft;
        o.scrollLeft += (sl > 0) ? -1 : 1;
        o.scrollLeft !== sl && (scrollX = scrollX || true);
        o.scrollLeft = sl;
    }
    return scrollX;
}

/*
    后端返回的账期数字转换成文字
*/
export function typeToName (type) {
    let dateLocale = {
        'en-US': {
            typeToName: {
                unspecified: 'Unspecified',
                month: 'Month',
                twoMonth: 'Two Month',
            }
        },
        'id-ID': {
            typeToName: {
                unspecified: 'Unspecified',
                month: 'Bulan',
                twoMonth: 'Dua Bulan',
            }
        }

    };
    let dateStyle = dateLocale[ getLocalLang() ];
    if (type === 0) {
        return dateStyle.typeToName.unspecified;
    } else if (type === 1) {
        return dateStyle.typeToName.month;
    } else if (type === 2) {
        return dateStyle.typeToName.twoMonth;
    }
}

/*
    后端返回的日期数字转换成时间
*/
export function changeDate (type) {
    if (type === 1) {
        return '1st';
    } else if (type === 2) {
        return '2nd';
    } else if (type === 3) {
        return '3rd';
    } else {
        return type + 'th';
    }
}

/**
 * 把数字转换成特定的格式(前缀货币或后缀单位)
 * @param value 值
 * @param symbol 货币符号单数
 * @param symbols 货币符号复数
 * @param isPerfix 前缀为true 后缀为false
 * @param thousandsSeparator 千位分隔符
 * @param decimalSeparatorPlaces 小数点位数
 * @param decimalSeparator 小数点分隔符
 * @returns {string}
 * (12345678): $ 12,345,678
 * (12345678,"HK$"): HK$ 12,345,678
 * (12345678,"€","€",true," ",",",2): $ 12,345,678.00
 */
export function numberFormat (value = 0, symbol, symbols, isPerfix = true, symbolSeparator = ' ', thousandsSeparator = '.', decimalSeparatorPlaces = 0, decimalSeparator = '.') {
    decimalSeparatorPlaces = !isNaN(decimalSeparatorPlaces = Math.abs(decimalSeparatorPlaces)) ? decimalSeparatorPlaces : 2;
    symbol = symbol !== undefined ? symbol : '$';
    symbols = symbols !== undefined ? symbols : symbol;

    let negative = value < 0 ? '-' : '',
        i = parseInt(value = Math.abs(+value || 0).toFixed(decimalSeparatorPlaces), 10) + '',
        j = (j = i.length) > 3 ? j % 3 : 0;
    let resultNum = [negative + (j ? i.substr(0, j) + thousandsSeparator : '') + i.substr(j).replace(/(\d{3})(?=\d)/g, '$1' + thousandsSeparator) +
    (decimalSeparatorPlaces ? decimalSeparator + Math.abs(value - i).toFixed(decimalSeparatorPlaces).slice(2) : '')];

    if (value > 1) {
        resultNum.push(symbols);
    } else {
        resultNum.push(symbol);
    }
    if (isPerfix) {
        return resultNum.reverse().join(symbolSeparator);
    } else {
        return resultNum.join(symbolSeparator);
    }
}

/* 视口的大小，部分移动设备浏览器对innerWidth的兼容性不好，需要
 *document.documentElement.clientWidth或者document.body.clientWidth
 *来兼容（混杂模式下对document.documentElement.clientWidth不支持）。
 *使用方法 ： getViewPort().width;
 */
export function getViewPort () {
    if (document.compatMode == 'BackCompat') {   // 浏览器嗅探，混杂模式
        return {
            width: document.body.clientWidth,
            height: document.body.clientHeight
        };
    } else {
        return {
            width: document.documentElement.clientWidth,
            height: document.documentElement.clientHeight
        };
    }
}

// 获得文档的大小（区别与视口）,与上面获取视口大小的方法如出一辙
export function getDocumentPort () {
    if (document.compatMode == 'BackCompat') {
        return {
            width: document.body.scrollWidth,
            height: document.body.scrollHeight
        };
    } else {
        return {
            width: Math.max(document.documentElement.scrollWidth, document.documentElement.clientWidth),
            height: Math.max(document.documentElement.scrollHeight, document.documentElement.clientHeight)
        };
    }
}


/**
 * 判断是不是移动端
 * @returns {boolean}
 */
export function isMobile() {
    var userAgentInfo = navigator.userAgent;
    const agents = ["Android", "iPhone", "SymbianOS", "Windows Phone", "iPad", "iPod"];
    for (let sys of agents) {
        if (userAgentInfo.indexOf(sys) > 0) {
            return true;
        }
    }
    return false;
}

/**
 * 判断是不是微信浏览器
 * @returns {boolean}
 */
export function isWechat() {
    var ua = navigator.userAgent.toLowerCase();
    return ua.match(/MicroMessenger/i) === "micromessenger";
}

/**
 * 判断浏览器所在机器操作系统版本
 */
export function getOsVersion(){
    var u = navigator.userAgent,version = '';
    if (u.indexOf('Mac OS X') > -1) {
        //ios
        var regStr_saf = /OS [\d._]*/gi;
        var verinfo = u.match(regStr_saf);
        version = (verinfo + "").replace(/[^0-9|_.]/ig,'').replace(/_/ig,'.');
    } else if (u.indexOf('Android') > -1
        || u.indexOf('Linux') > -1) {
        //android
        version = u.substr(u.indexOf('Android') + 8, u.indexOf(";", u.indexOf("Android")) - u.indexOf('Android') - 8);
    } else if (u.indexOf('BB10') > -1) {
        //黑莓bb10系统
        version = u.substr(u.indexOf('BB10') + 5, u.indexOf(";", u.indexOf("BB10")) - u.indexOf('BB10') - 5);
    } else if (u.indexOf('IEMobile')) {
        //windows phone
        version = u.substr(u.indexOf('IEMobile') + 9, u.indexOf(";", u.indexOf("IEMobile")) - u.indexOf('IEMobile') - 9);
    }
    return version;
}